@@ -3,14 +3,21 @@
 //! This library is an [`epi`] backend.
 //!
 //! If you are writing an app, you may want to look at [`eframe`](https://docs.rs/eframe) instead.
+//!
+//! ## Specifying the size of the egui canvas
+//! For performance reasons (on some browsers) the egui canvas does not, by default,
+//! fill the whole width of the browser.
+//! This can be changed by overriding [`epi::App::max_size_points`].
 
-#![cfg_attr(not(debug_assertions), deny(warnings))] // Forbid warnings in release builds
-#![deny(
-    rustdoc::broken_intra_doc_links,
-    rustdoc::invalid_codeblock_attributes,
-    rustdoc::missing_crate_level_docs,
-    rustdoc::private_intra_doc_links
-)]
+// Forbid warnings in release builds:
+#![cfg_attr(not(debug_assertions), deny(warnings))]
+// Disabled so we can support rust 1.51:
+// #![deny(
+//     rustdoc::broken_intra_doc_links,
+//     rustdoc::invalid_codeblock_attributes,
+//     rustdoc::missing_crate_level_docs,
+//     rustdoc::private_intra_doc_links
+// )]
 #![forbid(unsafe_code)]
 #![warn(clippy::all, rust_2018_idioms)]
 
@@ -87,6 +94,15 @@ pub fn native_pixels_per_point() -> f32 {
     }
 }
 
+pub fn prefer_dark_mode() -> Option<bool> {
+    Some(
+        web_sys::window()?
+            .match_media("(prefers-color-scheme: dark)")
+            .ok()??
+            .matches(),
+    )
+}
+
 pub fn canvas_element(canvas_id: &str) -> Option<web_sys::HtmlCanvasElement> {
     use wasm_bindgen::JsCast;
     let document = web_sys::window()?.document()?;
@@ -299,7 +315,7 @@ pub fn handle_output(output: &egui::Output, runner: &mut AppRunner) {
         copied_text,
         needs_repaint: _, // handled elsewhere
         events: _,        // we ignore these (TODO: accessibility screen reader)
-        text_cursor: cursor,
+        text_cursor_pos,
     } = output;
 
     set_cursor_icon(*cursor_icon);
@@ -315,9 +331,9 @@ pub fn handle_output(output: &egui::Output, runner: &mut AppRunner) {
     #[cfg(not(web_sys_unstable_apis))]
     let _ = copied_text;
 
-    if &runner.text_cursor != cursor {
-        move_text_cursor(cursor, runner.canvas_id());
-        runner.text_cursor = *cursor;
+    if &runner.last_text_cursor_pos != text_cursor_pos {
+        move_text_cursor(text_cursor_pos, runner.canvas_id());
+        runner.last_text_cursor_pos = *text_cursor_pos;
     }
 }
 
@@ -633,7 +649,8 @@ fn install_document_events(runner_ref: &AppRunnerRef) -> Result<(), JsValue> {
     }
 
     #[cfg(web_sys_unstable_apis)]
-    {
+    // paste is handled by IME text agent!
+    if false {
         // paste
         let runner_ref = runner_ref.clone();
         let closure = Closure::wrap(Box::new(move |event: web_sys::ClipboardEvent| {
@@ -1052,7 +1069,10 @@ fn install_canvas_events(runner_ref: &AppRunnerRef) -> Result<(), JsValue> {
             let delta = -scroll_multiplier
                 * egui::Vec2::new(event.delta_x() as f32, event.delta_y() as f32);
 
-            if event.ctrl_key() {
+            // Report a zoom event in case CTRL (on Windows or Linux) or CMD (on Mac) is pressed.
+            // This if-statement is equivalent to how `Modifiers.command` is determined in
+            // `modifiers_from_event()`, but we cannot directly use that fn for a `WheelEvent`.
+            if event.ctrl_key() || event.meta_key() {
                 runner_lock.input.raw.zoom_delta *= (delta.y / 200.0).exp();
             } else {
                 runner_lock.input.raw.scroll_delta += delta;
@@ -1115,12 +1135,13 @@ fn is_mobile() -> Option<bool> {
     Some(is_mobile)
 }
 
-// Move angnt to text cursor's position, on desktop/laptop, candidate window moves following text elemt(agent),
+// Move text agent to text cursor's position, on desktop/laptop,
+// candidate window moves following text element (agent),
 // so it appears that the IME candidate window moves with text cursor.
 // On mobile devices, there is no need to do that.
 fn move_text_cursor(cursor: &Option<egui::Pos2>, canvas_id: &str) -> Option<()> {
     let style = text_agent().style();
-    // Note: movint agent on mobile devices will lead to unpreditable scroll.
+    // Note: movint agent on mobile devices will lead to unpredictable scroll.
     if is_mobile() == Some(false) {
         cursor.as_ref().and_then(|&egui::Pos2 { x, y }| {
             let canvas = canvas_element(canvas_id)?;
